@@ -1,11 +1,10 @@
-#line 1 "inc/Module/Install/Can.pm - /usr/local/lib/perl5/site_perl/5.8.2/Module/Install/Can.pm"
-# $File: //depot/cpan/Module-Install/lib/Module/Install/Can.pm $ $Author: ingy $
-# $Revision: #5 $ $Change: 1377 $ $DateTime: 2003/03/20 15:11:54 $ vim: expandtab shiftwidth=4
+# $File: //depot/cpan/Module-Install/lib/Module/Install/Can.pm $ $Author: autrijus $
+# $Revision: #4 $ $Change: 1372 $ $DateTime: 2003/03/18 11:18:27 $ vim: expandtab shiftwidth=4
 
 package Module::Install::Can;
 use Module::Install::Base; @ISA = qw(Module::Install::Base);
+
 $VERSION = '0.01';
-use strict;
 
 # check if we can run some command
 sub can_run {
@@ -15,8 +14,7 @@ sub can_run {
     require File::Spec;
     require ExtUtils::MakeMaker;
 
-    my $_cmd = $cmd;
-    return $_cmd if (-x $_cmd or $_cmd = MM->maybe_command($_cmd));
+    return $cmd if (-x $cmd or $cmd = MM->maybe_command($cmd));
 
     for my $dir ((split /$Config::Config{path_sep}/, $ENV{PATH}), '.') {
         my $abs = File::Spec->catfile($dir, $_[1]);
